/*  Lyrics plugin for XMMS by Karol Bryd <kbryd@albedo.art.pl>
 *  X11amp is Copyright (C) 1998-1999  Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <xmms/configfile.h>
#include <xmms/xmmsctrl.h>
#include <xmms/plugin.h>

#include "lyrics.h"
#include "list.h"
#include "creator.h"

#include "config.h"
#include "support.h"

enum
{
    HUE,
    SATURATION,
    VALUE,
    RED,
    GREEN,
    BLUE
};

GtkWidget *conf_win = NULL;

static GtkWidget *filew, *font, *font_browser;
static GtkWidget *string, *csd;
static GtkWidget *change_active_color_button, *change_bg_color_button,

    *change_fg_color_button;

GtkWidget *preview_active_color = NULL, *preview_fg_color =
    NULL, *preview_bg_color = NULL;

void
update_prefs(void)
{
    ConfigFile *cfg;
    gchar *filename;
    gchar tmp[32];

    filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
    cfg = xmms_cfg_open_file(filename);
    if (cfg)
    {
	xmms_cfg_write_string(cfg, "lyrics", "lyrics_path", lyrics_dir_path);
	xmms_cfg_write_string(cfg, "lyrics", "lyrics_font", lyrics_font);
	sprintf(tmp, "%d", width);
	xmms_cfg_write_string(cfg, "lyrics", "width", tmp);
	sprintf(tmp, "%d", height);
	xmms_cfg_write_string(cfg, "lyrics", "height", tmp);

	sprintf(tmp, "rgb:%02x/%02x/%02x", active.red, active.green,
		active.blue);
	xmms_cfg_write_string(cfg, "lyrics", "active_color", tmp);

	sprintf(tmp, "rgb:%02x/%02x/%02x", background.red, background.green,
		background.blue);
	xmms_cfg_write_string(cfg, "lyrics", "bg_color", tmp);

	sprintf(tmp, "rgb:%02x/%02x/%02x", foreground.red, foreground.green,
		foreground.blue);
	xmms_cfg_write_string(cfg, "lyrics", "fg_color", tmp);
	xmms_cfg_write_int(cfg, "lyrics", "hide_mode", hide_mode);
	xmms_cfg_write_int(cfg, "lyrics", "look_in_mp3dir_mode", look_in_mp3dir_mode);
	xmms_cfg_write_int(cfg, "lyrics", "recursive_mode", recursive_mode);
	xmms_cfg_write_int(cfg, "lyrics", "desperate_mode", desperate_mode);
	xmms_cfg_write_int(cfg, "lyrics", "fuzzy_percent", fuzzy_percent);

	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);
    }
}

void
update_preview_widgets()
{
    guchar buf[60 * 3];
    gint y, x, i;

    if (!preview_active_color || !preview_bg_color || !preview_fg_color)
	return;

    for (x = y = 0; y < 20; y++)
    {
	x = 0;
	for (i = 0; i < 60; i++)
	{
	    buf[x + 0] = active.red;
	    buf[x + 1] = active.green;
	    buf[x + 2] = active.blue;
	    x += 3;
	}
	gtk_preview_draw_row(GTK_PREVIEW(preview_active_color), buf, 0,
			     y, 60);
    }

    gtk_widget_draw(preview_active_color, NULL);

    for (x = y = 0; y < 20; y++)
    {
	x = 0;
	for (i = 0; i < 60; i++)
	{
	    buf[x + 0] = background.red;
	    buf[x + 1] = background.green;
	    buf[x + 2] = background.blue;
	    x += 3;
	}
	gtk_preview_draw_row(GTK_PREVIEW(preview_bg_color), buf, 0, y, 60);
    }

    gtk_widget_draw(preview_bg_color, NULL);

    for (x = y = 0; y < 20; y++)
    {
	x = 0;
	for (i = 0; i < 60; i++)
	{
	    buf[x + 0] = foreground.red;
	    buf[x + 1] = foreground.green;
	    buf[x + 2] = foreground.blue;
	    x += 3;
	}
	gtk_preview_draw_row(GTK_PREVIEW(preview_fg_color), buf, 0, y, 60);
    }

    gtk_widget_draw(preview_fg_color, NULL);
}

void
file_ok_sel(GtkWidget * w, GtkFileSelection * fs)
{
    strcpy(lyrics_dir_path,
	   gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

    gtk_entry_set_text(GTK_ENTRY(string), lyrics_dir_path);
    update_prefs();
    gtk_widget_destroy(filew);
}

void
close_font(void *args)
{
    gtk_widget_destroy(font_browser);
}

void
close_file(void *args)
{
    gtk_widget_destroy(filew);
}

void *
select_drawer(void *args)
{
    filew = gtk_file_selection_new(_("Select lyrics directory"));

    gtk_signal_connect(GTK_OBJECT
		       (GTK_FILE_SELECTION(filew)->ok_button),
		       "clicked", (GtkSignalFunc) file_ok_sel, filew);
    gtk_signal_connect(GTK_OBJECT
		       (GTK_FILE_SELECTION(filew)->cancel_button),
		       "clicked", GTK_SIGNAL_FUNC(close_file), NULL);

    gtk_widget_show(filew);
}

void *
font_ok_sel(GtkWidget * fontb)
{
    strcpy(lyrics_font,
	   gtk_font_selection_dialog_get_font_name
	   (GTK_FONT_SELECTION_DIALOG(font_browser)));
    gtk_entry_set_text(GTK_ENTRY(font), lyrics_font);
    update_prefs();

    current_font = gdk_font_load(lyrics_font);
    if (current_font)
    {
	redraw_list();
	gtk_widget_destroy(font_browser);
    }
}

void *
select_font(void *args)
{
    font_browser = gtk_font_selection_dialog_new(_("Select display font"));
    gtk_signal_connect(GTK_OBJECT
		       (GTK_FONT_SELECTION_DIALOG(font_browser)->ok_button),
		       "clicked", GTK_SIGNAL_FUNC(font_ok_sel), font_browser);
    gtk_signal_connect(GTK_OBJECT
		       (GTK_FONT_SELECTION_DIALOG
			(font_browser)->cancel_button), "clicked",
		       GTK_SIGNAL_FUNC(close_font), NULL);
    gtk_widget_show(font_browser);
}

void *
close_conf(void *args)
{
    update_prefs();
    if (conf_win)
    {
	gtk_widget_destroy(conf_win);
	conf_win = NULL;
    }
}

#if 0
void update_hide_mode(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active)
		hide_mode = 1;
	else
		hide_mode = 0;
}

void update_recursive_mode(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active)
		recursive_mode = 1;
	else
		recursive_mode = 0;
}

void update_desperate_mode(GtkWidget *widget, gpointer data)
{
	if(GTK_TOGGLE_BUTTON(widget)->active)
		desperate_mode = 1;
	else
		desperate_mode = 0;
}
#endif

void update_checkbox(GtkWidget *widget, gpointer data)
{
	dprintf("data\t%p\n", data);
	if(GTK_TOGGLE_BUTTON(widget)->active)
		*(gint *)data = 1;
	else
		*(gint *)data = 0;
	dprintf("\thide\t%d (%p)\n"
		"\tmp3\t%d (%p)\n"
		"\tsubd\t%d (%p)\n",
		hide_mode, &hide_mode, look_in_mp3dir_mode, &look_in_mp3dir_mode, recursive_mode, &recursive_mode);
}

void update_fuzzy_percent(GtkWidget *widget, gpointer date)
{
	fuzzy_percent = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
}

void
update_preview_callback(GtkWidget * w1, GtkWidget * w2)
{
    GtkColorSelection *colorsel;
    gdouble color[8];

    if (!w2)
    {
	gtk_widget_destroy(csd);
	return;
    }

    colorsel =
	GTK_COLOR_SELECTION(((GtkColorSelectionDialog *) (csd))->colorsel);
    gtk_color_selection_get_color(colorsel, color);
    colormap = gdk_colormap_get_system();

    if (w2 == change_active_color_button)
    {
	gdk_colormap_free_colors(colormap, &active, 1);
	bzero(&active, sizeof(GdkColor));
	active.red = (int) (color[0] * 255.0) * 257;
	active.green = (int) (color[1] * 255.0) * 257;
	active.blue = (int) (color[2] * 255.0) * 257;
	gdk_color_alloc(colormap, &active);
    }
    else if (w2 == change_bg_color_button)
    {
	gdk_colormap_free_colors(colormap, &background, 1);
	bzero(&background, sizeof(GdkColor));
	background.red = (int) (color[0] * 255.0) * 257;
	background.green = (int) (color[1] * 255.0) * 257;
	background.blue = (int) (color[2] * 255.0) * 257;
	gdk_color_alloc(colormap, &background);
    }
    else
    {
	gdk_colormap_free_colors(colormap, &foreground, 1);
	bzero(&foreground, sizeof(GdkColor));
	foreground.red = (int) (color[0] * 255.0) * 257;
	foreground.green = (int) (color[1] * 255.0) * 257;
	foreground.blue = (int) (color[2] * 255.0) * 257;
	gdk_color_alloc(colormap, &foreground);
    }

    redraw_list();
    update_preview_widgets();
    gtk_widget_destroy(csd);
}

void
set_color(GtkWidget * widget, char *title)
{
    csd = gtk_color_selection_dialog_new(title);
    gtk_window_set_modal(GTK_WINDOW(csd), TRUE);
    gtk_window_set_transient_for(GTK_WINDOW(csd), GTK_WINDOW(conf_win));

    gtk_widget_show(csd);
    gtk_signal_connect(GTK_OBJECT(csd), "destroy",
		       GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(csd));

    gtk_signal_connect(GTK_OBJECT
		       (GTK_COLOR_SELECTION_DIALOG(csd)->cancel_button),
		       "clicked", GTK_SIGNAL_FUNC(update_preview_callback),
		       NULL);

    gtk_signal_connect(GTK_OBJECT
		       (GTK_COLOR_SELECTION_DIALOG(csd)->ok_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(update_preview_callback), widget);
}

void
configure(void)
{
    GtkWidget *button, *hbox, *vbox, *frame_path, *frame_lyrics_font, *hbox2;
    GtkWidget *exit_button, *font_button, *frame_hbox, *creator_button;
    GtkWidget *colors_frame, *color_vbox, *color_hbox;
    GtkWidget *frame_misc, *vbox_misc, *hide, *hide_always;
    GtkWidget *look_in_mp3dir, *recursive;
    GtkWidget *frame_fuzzy, *vbox_fuzzy, *desperate, *hbox_fuzzy, *label_fuzzy, *spin_fuzzy;
    GtkTooltips *tooltips;

    /* Yes, it doubles code found in list.c, but it is required. The reason is
     * (I guess) that this code and plugin itself are running in separate
     * threads. */
    bindtextdomain(PACKAGE, LOCALEDIR);

    if (!conf)
    {
	ConfigFile *cfg;
	GtkStyle *style;
	gchar *filename, *tmp;

	conf = TRUE;
	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);

	bzero(lyrics_font, sizeof(lyrics_font));
	bzero(lyrics_dir_path, sizeof(lyrics_dir_path));
	bzero(last_file, sizeof(last_file));

	cfg = xmms_cfg_open_file(filename);
	if (!cfg)
	    cfg = xmms_cfg_new();

	if (xmms_cfg_read_string(cfg, "lyrics", "lyrics_path", &tmp))
	    strcpy(lyrics_dir_path, tmp);
	if (xmms_cfg_read_string(cfg, "lyrics", "lyrics_font", &tmp))
	    strcpy(lyrics_font, tmp);
	if (xmms_cfg_read_string(cfg, "lyrics", "width", &tmp))
	    width = atoi(tmp);
	else
	    width = DEFAULT_WIDTH;

	if (xmms_cfg_read_string(cfg, "lyrics", "height", &tmp))
	    height = atoi(tmp);
	else
	    height = DEFAULT_HEIGHT;

	if (!
	    (xmms_cfg_read_string
	     (cfg, "lyrics", "active_color", &active_color)))
	    active_color = g_strdup("lightblue");

	if (!(xmms_cfg_read_string(cfg, "lyrics", "bg_color", &bg_color)))
	    bg_color = g_strdup("white");

	if (!(xmms_cfg_read_string(cfg, "lyrics", "fg_color", &fg_color)))
	    fg_color = g_strdup("black");

	if (!(xmms_cfg_read_int(cfg, "lyrics", "hide_mode", &hide_mode)))
	    hide_mode = 1;

	if (!(xmms_cfg_read_int(cfg, "lyrics", "look_in_mp3dir_mode", &look_in_mp3dir_mode)))
	    hide_mode = 1;

	if (!(xmms_cfg_read_int(cfg, "lyrics", "recursive_mode", &recursive_mode)))
	    hide_mode = 1;

	if (!(xmms_cfg_read_int(cfg, "lyrics", "desperate_mode", &desperate_mode)))
	    desperate_mode = 1;
	
	if (!(xmms_cfg_read_int(cfg, "lyrics", "fuzzy_percent", &fuzzy_percent)) ||
            fuzzy_percent > 100 ||
	    fuzzy_percent < 0)
	    fuzzy_percent = 20;

	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);

	colormap = gdk_colormap_get_system();
	if (colormap)
	{
	    gdk_color_parse(active_color, &active);
	    gdk_color_alloc(colormap, &active);
	    gdk_color_parse(bg_color, &background);
	    gdk_color_alloc(colormap, &background);
	    gdk_color_parse(fg_color, &foreground);
	    gdk_color_alloc(colormap, &foreground);
	}
    }

    tooltips = gtk_tooltips_new();

    conf_win = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_signal_connect(GTK_OBJECT(conf_win),
		       "destroy", GTK_SIGNAL_FUNC(close_conf), NULL);

    gtk_window_set_title(GTK_WINDOW(conf_win), _("Lyrics plugin configuration"));
    gtk_container_border_width(GTK_CONTAINER(conf_win), 5);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(conf_win), vbox);
    gtk_widget_show(vbox);

    colors_frame = gtk_frame_new(_("Colors:"));
    gtk_container_set_border_width(GTK_CONTAINER(colors_frame), 5);
    gtk_box_pack_start(GTK_BOX(vbox), colors_frame, TRUE, TRUE, 0);

    color_vbox = gtk_vbox_new(TRUE, 10);
    gtk_container_add(GTK_CONTAINER(colors_frame), color_vbox);
    gtk_container_set_border_width(GTK_CONTAINER(color_vbox), 5);
    gtk_widget_show(color_vbox);

    gtk_widget_show(colors_frame);

    color_hbox = gtk_hbox_new(TRUE, 10);
    gtk_container_add(GTK_CONTAINER(color_vbox), color_hbox);
    gtk_container_set_border_width(GTK_CONTAINER(color_hbox), 5);
    gtk_widget_show(color_hbox);

    change_active_color_button = gtk_button_new_with_label(_("Active  "));
    gtk_container_add(GTK_CONTAINER(color_hbox), change_active_color_button);
    gtk_signal_connect(GTK_OBJECT(change_active_color_button), "clicked",
		       GTK_SIGNAL_FUNC(set_color), _("Pick \"active\" color"));

    gtk_widget_show(change_active_color_button);

    preview_active_color = gtk_preview_new(GTK_PREVIEW_COLOR);
    gtk_preview_size(GTK_PREVIEW(preview_active_color), 60, 20);
    gtk_container_add(GTK_CONTAINER(color_hbox), preview_active_color);

    gtk_widget_show(preview_active_color);

    color_hbox = gtk_hbox_new(TRUE, 10);
    gtk_container_add(GTK_CONTAINER(color_vbox), color_hbox);
    gtk_container_set_border_width(GTK_CONTAINER(color_hbox), 5);
    gtk_widget_show(color_hbox);

    change_bg_color_button = gtk_button_new_with_label(_("Background"));
    gtk_container_add(GTK_CONTAINER(color_hbox), change_bg_color_button);
    gtk_widget_show(change_bg_color_button);
    gtk_signal_connect(GTK_OBJECT(change_bg_color_button), "clicked",
		       GTK_SIGNAL_FUNC(set_color),
		       _("Pick \"background\" color"));

    preview_bg_color = gtk_preview_new(GTK_PREVIEW_COLOR);
    gtk_preview_size(GTK_PREVIEW(preview_bg_color), 60, 20);
    gtk_container_add(GTK_CONTAINER(color_hbox), preview_bg_color);

    gtk_widget_show(preview_bg_color);

    color_hbox = gtk_hbox_new(TRUE, 10);
    gtk_container_add(GTK_CONTAINER(color_vbox), color_hbox);
    gtk_container_set_border_width(GTK_CONTAINER(color_hbox), 5);
    gtk_widget_show(color_hbox);

    change_fg_color_button = gtk_button_new_with_label(_("Foreground"));
    gtk_container_add(GTK_CONTAINER(color_hbox), change_fg_color_button);
    gtk_widget_show(change_fg_color_button);
    gtk_signal_connect(GTK_OBJECT(change_fg_color_button), "clicked",
		       GTK_SIGNAL_FUNC(set_color),
		       _("Pick \"foreground\" color"));

    preview_fg_color = gtk_preview_new(GTK_PREVIEW_COLOR);
    gtk_preview_size(GTK_PREVIEW(preview_fg_color), 60, 20);
    gtk_container_add(GTK_CONTAINER(color_hbox), preview_fg_color);

    gtk_widget_show(preview_fg_color);

    gtk_widget_show(color_vbox);

    frame_misc = gtk_frame_new(_("Miscellaneous:"));
    gtk_container_set_border_width(GTK_CONTAINER(frame_misc), 5);
    gtk_box_pack_start(GTK_BOX(vbox), frame_misc, TRUE, TRUE, 0);

    vbox_misc = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame_misc), vbox_misc);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_misc), 5);
    gtk_widget_show(vbox_misc);

    gtk_widget_show(frame_misc);

	hide = gtk_check_button_new_with_label(_("Hide window when not playing"));
	gtk_container_add(GTK_CONTAINER(vbox_misc), hide);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hide), hide_mode);
	gtk_widget_show(hide);

    gtk_signal_connect(GTK_OBJECT(hide), "toggled",
		       GTK_SIGNAL_FUNC(update_checkbox), &hide_mode);
	
        look_in_mp3dir = gtk_check_button_new_with_label(_("Look for lyrics in mp3 directory"));
	gtk_container_add(GTK_CONTAINER(vbox_misc), look_in_mp3dir);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(look_in_mp3dir), look_in_mp3dir_mode);
	gtk_widget_show(look_in_mp3dir);
	gtk_tooltips_set_tip(tooltips, look_in_mp3dir, _(
			"If you want plugin to look for lyrics also in directory containing "
			"file being played check this box.\n"
			"Be aware that more directories plugin has to check, more time it "
			"takes."), _(
			"Private TIP"));

    gtk_signal_connect(GTK_OBJECT(look_in_mp3dir), "toggled",
		       GTK_SIGNAL_FUNC(update_checkbox), &look_in_mp3dir_mode);

        recursive = gtk_check_button_new_with_label(_("Look for lyrics in subdirecotries"));
	gtk_container_add(GTK_CONTAINER(vbox_misc), recursive);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(recursive), recursive_mode);
	gtk_widget_show(recursive);
	gtk_tooltips_set_tip(tooltips, recursive, _(
			"This plugin can look for lyrics file in subdirectories and this "
			"checkbox controls this feature.\n"
			"Looking in subdirectories can significantly incrase amount of "
			"time spent looking for lyrics, so don't check this option if "
			"you don't need it."), _(
			"Private TIP"));

    gtk_signal_connect(GTK_OBJECT(recursive), "toggled",
		       GTK_SIGNAL_FUNC(update_checkbox), &recursive_mode);
    
    frame_fuzzy = gtk_frame_new(_("Fuzzy search settings:"));
    gtk_container_set_border_width(GTK_CONTAINER(frame_fuzzy), 5);
    gtk_box_pack_start(GTK_BOX(vbox), frame_fuzzy, TRUE, TRUE, 0);

    vbox_fuzzy = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame_fuzzy), vbox_fuzzy);
    gtk_container_set_border_width(GTK_CONTAINER(vbox_fuzzy), 5);
    gtk_widget_show(vbox_fuzzy);

    gtk_widget_show(frame_fuzzy);

	desperate = gtk_check_button_new_with_label(_("Ignore artist name as last resort"));
	gtk_container_add(GTK_CONTAINER(vbox_fuzzy), desperate);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(desperate), desperate_mode);
	gtk_widget_show(desperate);
	gtk_tooltips_set_tip(tooltips, desperate, _(
			"At first this plugin tries to find lyrics file "
			"with same name as audio file. This works well, if "
			"your audio and lyrics files have same names. "
			"But different people uses different conventions.\n"
			"If this option is selected, this plugin will try to "
			"match lyrics file using only title and ignoring arists "
			"name.\n"
			"But be aware, that this can result in mistakes, if you "
			"have two files with same titles but of different artists."), _(
			"Private TIP"));

    gtk_signal_connect(GTK_OBJECT(desperate), "toggled",
		       GTK_SIGNAL_FUNC(update_checkbox), &desperate_mode);

    hbox_fuzzy = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(vbox_fuzzy), hbox_fuzzy);
    gtk_container_set_border_width(GTK_CONTAINER(hbox_fuzzy), 5);
    gtk_widget_show(hbox_fuzzy);

        label_fuzzy = gtk_label_new(_("Fuzzines (%)"));
	gtk_container_add(GTK_CONTAINER(hbox_fuzzy), label_fuzzy);
	gtk_widget_show(label_fuzzy);

        spin_fuzzy = gtk_spin_button_new(
		       GTK_ADJUSTMENT(gtk_adjustment_new(fuzzy_percent, 0, 100, 1, 10, 10)),
		       1, 0);
	gtk_container_add(GTK_CONTAINER(hbox_fuzzy), spin_fuzzy);
	gtk_widget_show(spin_fuzzy);
	gtk_tooltips_set_tip(tooltips, spin_fuzzy, _(
			"This value tells plugin how different can be name of "
			"lyrics file from name of audio file. Great thing, if "
			"you don't want to waste your time renaming files ;)\n"
			"Default value is 20%"), _(
			"Private TIP"));

    gtk_signal_connect(GTK_OBJECT(spin_fuzzy), "changed",
		       GTK_SIGNAL_FUNC(update_fuzzy_percent), NULL);

    frame_path = gtk_frame_new(_("Path to the lyrics directory:"));
    gtk_container_set_border_width(GTK_CONTAINER(frame_path), 5);
    gtk_box_pack_start(GTK_BOX(vbox), frame_path, TRUE, TRUE, 0);

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame_path), hbox);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
    gtk_widget_show(hbox);

    gtk_widget_show(frame_path);

    string = gtk_entry_new_with_max_length(255);
    gtk_container_add(GTK_CONTAINER(hbox), string);
    gtk_entry_set_text(GTK_ENTRY(string), lyrics_dir_path);
    gtk_widget_show(string);

    button = gtk_button_new_with_label(_("Select..."));
    gtk_container_add(GTK_CONTAINER(hbox), button);
    gtk_widget_show(button);

    frame_lyrics_font = gtk_frame_new(_("Lyrics font:"));
    gtk_container_set_border_width(GTK_CONTAINER(frame_lyrics_font), 5);
    gtk_box_pack_start(GTK_BOX(vbox), frame_lyrics_font, TRUE, TRUE, 0);
    gtk_widget_show(frame_lyrics_font);

    frame_hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(frame_lyrics_font), frame_hbox);
    gtk_container_set_border_width(GTK_CONTAINER(frame_hbox), 5);
    gtk_widget_show(frame_hbox);

    font = gtk_entry_new_with_max_length(255);
    gtk_container_add(GTK_CONTAINER(frame_hbox), font);
    gtk_entry_set_text(GTK_ENTRY(font), lyrics_font);
    gtk_widget_show(font);

    font_button = gtk_button_new_with_label(_("Browse..."));
    gtk_container_add(GTK_CONTAINER(frame_hbox), font_button);
    gtk_widget_show(font_button);

    gtk_signal_connect(GTK_OBJECT(font_button), "clicked",
		       GTK_SIGNAL_FUNC(select_font), NULL);

    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(select_drawer), NULL);

    creator_button = gtk_button_new_with_label(_("Create Lyrics"));
    gtk_container_add(GTK_CONTAINER(vbox), creator_button);
    gtk_signal_connect(GTK_OBJECT(creator_button), "clicked",
		       GTK_SIGNAL_FUNC(create_lyrics), NULL);

    exit_button = gtk_button_new_with_label(_("Exit"));
    gtk_container_add(GTK_CONTAINER(vbox), exit_button);
    gtk_signal_connect(GTK_OBJECT(exit_button), "clicked",
		       GTK_SIGNAL_FUNC(close_conf), NULL);

    gtk_widget_show(creator_button);
    gtk_widget_show(exit_button);
    gtk_widget_show(conf_win);

    update_preview_widgets();
}
