/*  Lyrics plugin for X11amp by Karol Bryd <kbryd@albedo.com.pl>
 *  X11amp is Copyright (C) 1998-1999  Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include "lyrics.h"

static struct Song *
init_song(void)
{
    struct Song *song;

    song = malloc(sizeof(struct Song));

    song->head = song->tail = NULL;
    return (song);
}

static void
addline(struct Song *song, struct LyricsLine *node)
{
    node->next = 0;

    if (song->tail)
    {
	song->tail->next = node;
	song->tail = node;
    }
    else
    {
	song->head = node;
	song->tail = node;
    }
}

static char *
get_line_body(char *line)
{
    int a = 0;

    while (line[a] != 0)
    {
	if (line[a] == ']')
	{
	    if (line[a + 1] != '[')
		return (&line[a + 1]);
	}
	a++;
    }
}

struct LyricsLine *
get_lyrics_line(char *buffer, struct Song *song, int current_line)
{
    struct LyricsLine *newll;
    int current_pos = 0;
    int minutes, seconds;

    while (1)
    {
	if (buffer[current_pos] != '[')
	    break;

	newll = calloc(1, sizeof(struct LyricsLine));

	if (!newll)
	    return (NULL);

	sscanf(&buffer[current_pos], "[%ld:%ld]", &minutes, &seconds);
	newll->line_time = minutes * 60 + seconds;
	newll->line_number = current_line;
	newll->buffer = strdup(get_line_body(buffer));
	newll->rep = current_pos ? 1 : 0;
	addline(song, newll);

	while (buffer[current_pos++] != ']')
	    ;
    }

    return (newll);
}

struct Song *
read_lyrics_file(char *filename)
{
    struct LyricsLine *line;
    struct Song *song;
    FILE *file;
    char buffer[255];
    char *x;
    int current_line = 0;

    file = fopen(filename, "r");
    if (!file)
	return (NULL);

    song = init_song();

    while (fgets(buffer, sizeof(buffer), file))
    {
	if (x = strchr(buffer, '\r'))
	    *x = 0;

	if ((buffer[0] == '[') && (isdigit(buffer[1])))
	{
	    line = get_lyrics_line(buffer, song, current_line);
	    current_line++;
	}
	else if ((buffer[0] == '[') && !(isdigit(buffer[1])))
	{
	    if ((buffer[1] == 'a') && (buffer[2] == 'r'))	// artist
		song->artist = strdup(&buffer[3]);
	    else if ((buffer[1] == 't') && (buffer[2] == 'i'))	// title
		song->title = strdup(&buffer[3]);
	    else if ((buffer[1] == 'l') && (buffer[2] == 'a'))	// language
		song->language = strdup(&buffer[3]);
	    else if ((buffer[1] == 'b') && (buffer[2] == 'y'))	// author
		song->author = strdup(&buffer[3]);
	}
	else
	{
	    struct LyricsLine *ll;

	    ll = calloc(1, sizeof(struct LyricsLine));

	    ll->buffer = strdup(buffer);
	    ll->line_time = -1;
	    ll->rep = 0;
	    ll->line_number = current_line;
	    addline(song, ll);
	    current_line++;
	}
    }
    return (song);
}

static char *
show_lyrics(struct Song *song, int time)
{
    struct LyricsLine *line;

    line = song->head;

    while (line->next)
    {
	if (line->line_time == time)
	    return (line->buffer);
	line = line->next;
    }
}

/*
   int main(void)
   {
   struct Song *song;
   int a = 0;

   song = read_lyrics_file("./test_lyrics");
   printf("begin\n");

   for(a = 0; a < 3 * 60 + 30; a++)
   {
   show_lyrics(song, a);
   }
   }
 */
